@@ -28,16 +28,16 @@ class ParameterizingString(unicode):
     """
 
     def __new__(cls, *args):
-        """P.__new__(cls, capname, [normal, [name]])
+        """P.__new__(cls, cap, [normal, [name]])
 
-        :arg capname: parameterized string suitable for curses.tparm()
+        :arg cap: parameterized string suitable for curses.tparm()
         :arg normal: terminating sequence for this capability.
         :arg name: name of this terminal capability.
         """
         assert len(args) and len(args) < 4, args
         new = unicode.__new__(cls, args[0])
-        new._normal = len(args) >= 2 and args[1] or u''
-        new._name = len(args) == 3 and args[2] or u'<not specified>'
+        new._normal = len(args) > 1 and args[1] or u''
+        new._name = len(args) > 2 and args[2] or u'<not specified>'
         return new
 
     def __call__(self, *args):
@@ -160,10 +160,13 @@ def split_compound(compound):
 
 def resolve_capability(term, attr):
     """Return a Unicode string for the terminal capability ``attr``,
-    or an empty string if not found.
+    or an empty string if not found, or if terminal is without styling
+    capabilities.
     """
     # Decode sequences as latin1, as they are always 8-bit bytes, so when
     # b'\xff' is returned, this must be decoded to u'\xff'.
+    if not term.does_styling:
+        return u''
     val = curses.tigetstr(term._sugar.get(attr, attr))
     return u'' if val is None else val.decode('latin1')
 
